declare module BA.Ui.Types {
    /**
     *  This type creates a string literal type that consists of all comma seperated permutations of all given single
     *  string literal types. Mostly used with lists of predefined parameter values.
     *
     *  Example:
     *      let exa: Permutations<'type_a' | 'type_b'>;
     *
     *  will allow as values for exa:
     *  - 'type_a'
     *  - 'type_b'
     *  - 'type_a,type_b'
     *  - 'type_b,type_a'
     */
    type Permutations<T extends string, U extends string = T> = T extends any ? (T | `${T},${Permutations<Exclude<U, T>>}`) : never;
    /**
     * Will create a type of a given type where one of the given properties is mandatory (any). Useful if you have some
     * type with two optional parameters but eventully you NEED one of these (but no matter which one).
     *
     * Example:
     *      interface BaseType {
     *          caption?: string;
     *          nulltext?: string;
     *      };
     *
     *      type MyType = RequireOnlyOne<BaseType, 'caption' | 'nulltext'>;
     *
     * The compiler now will raise an error, if you create an object of MyType with none of the given properties set,
     * even though both properties are optional in the base type.
     */
    type RequireOnlyOne<T, Keys extends keyof T = keyof T> = Pick<T, Exclude<keyof T, Keys>> & {
        [K in Keys]-?: Required<Pick<T, K>> & Partial<Record<Exclude<Keys, K>, undefined>>;
    }[Keys];
}
